@@ -17,3 +17,5 @@ from .vqe_solver import VQESolver, BuiltInAnsatze
 from .sa_vqe_solver import SA_VQESolver
 from .sa_oo_vqe_solver import SA_OO_Solver
 from .iqcc_solver import iQCC_solver
+from .iqcc_ilc_solver import iQCC_ILC_solver
+from .tetris_adapt_vqe_solver import TETRISADAPTSolver
